/**
* Copyright 2021 CriticalBlue Ltd.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
* associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or
* substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
* NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
* OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#import "ApproovDiscoveryPlugin.h"
#import "Approov/Approov.h"

// Basic plugin implementation to initialize the SDK, fetch an updated dynamic config and to fetch an Approov token
@implementation ApproovDiscoveryPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"approov_discovery"
            binaryMessenger:[registrar messenger]];
  ApproovHttpClientPlugin* instance = [[ApproovHttpClientPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    if ([@"initialize" isEqualToString:call.method]) {
        NSError* error = nil;
        NSString *initialConfig = nil;
        if (call.arguments[@"initialConfig"] != [NSNull null]) initialConfig = call.arguments[@"initialConfig"];
        NSString *dynamicConfig = nil;
        if (call.arguments[@"dynamicConfig"] != [NSNull null]) dynamicConfig = call.arguments[@"dynamicConfig"];
        NSString *comment = nil;
        if (call.arguments[@"comment"] != [NSNull null]) comment = call.arguments[@"comment"];
        [Approov initialize:initialConfig updateConfig:dynamicConfig comment:comment error:&error];
        if (error == nil) {
            [Approov setUserProperty:call.arguments[@"discovery"]];
            result(nil);
        } else {
            result([FlutterError errorWithCode:[NSString stringWithFormat:@"%ld", (long)error.code]
                message:error.domain details:error.localizedDescription]);
        }
    } else if ([@"fetchConfig" isEqualToString:call.method]) {
        result([Approov fetchConfig]);
    } else if ([@"fetchApproovTokenAndWait" isEqualToString:call.method]) {
        ApproovTokenFetchResult *tokenFetchResult = [Approov fetchApproovTokenAndWait:call.arguments[@"url"]];
        NSMutableDictionary *tokenFetchResultMap = [NSMutableDictionary dictionary];
        tokenFetchResultMap[@"IsConfigChanged"] = [NSNumber numberWithBool:tokenFetchResult.isConfigChanged];
        tokenFetchResultMap[@"LoggableToken"] = tokenFetchResult.loggableToken;
        result((NSDictionary*)tokenFetchResultMap);
    } else {
        result(FlutterMethodNotImplemented);
    }
}

@end
